<?php

namespace App\Http\Controllers\Api;

use Illuminate\Support\Facades\Lang;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Request;
use Illuminate\Pagination\LengthAwarePaginator;
use League\Fractal\Manager;
use League\Fractal\Resource\Item;
use League\Fractal\Resource\Collection;
use League\Fractal\Serializer\ArraySerializer;
use League\Fractal\Pagination\IlluminatePaginatorAdapter;

class ApiBaseController extends Controller
{
    const RESPONSE_STATUS_OK = 200;
    const RESPONSE_STATUS_ERROR = 400;

    const LIMIT = 10;

    protected $statusCode;

    public function __construct(Manager $fractal)
    {
        $this->fractal = $fractal;
        $this->fractal->setSerializer(new ArraySerializer());
        // Are we going to try and include embedded data?
        if (Request::has('embed')) {
            $this->fractal->parseIncludes(Request::get('embed'));
        }

    }

    protected function setStatusCode($statusCode)
    {
        $this->statusCode = $statusCode;
        return $this;
    }

    protected function getStatusCode()
    {
        return $this->statusCode;
    }

    protected function respond($data, $headers = [])
    {
        return response()->json($data, $this->getStatusCode(), $headers);
    }

    protected function getData($collection, $callback)
    {
        $resource = new Collection($collection, $callback);
        $rootScope = $this->fractal->createData($resource);
        return $rootScope->toArray();
    }

    protected function respondWithData($data, $statusCode = null, array $headers = [])
    {
        $statusCode = !empty($statusCode) ? $statusCode : self::RESPONSE_STATUS_OK;
        $data = ['status' => $statusCode, 'data' => $data];
        return $this->setStatusCode($statusCode)->respond($data, $headers);
    }

    public function respondWithArray($data, $statusCode = null, array $headers = [])
    {
        $statusCode = !empty($statusCode) ? $statusCode : self::RESPONSE_STATUS_OK;
        $data['status'] = $statusCode;
        return $this->setStatusCode($statusCode)->respond($data, $headers);
    }
    public function respondWithSomeData($data)
    {
        $statusCode=200;
        return $this->setStatusCode($statusCode)->respond($data);
    }

    protected function respondWithItem($item, $callback)
    {
        $resource = new Item($item, $callback);
        $rootScope = $this->fractal->createData($resource);
        return $this->respondWithData($rootScope->toArray());
    }

    protected function respondWithCollection($collection, $callback)
    {
        $resource = new Collection($collection, $callback);
        if ($collection instanceof LengthAwarePaginator) {
            $resource->setPaginator(new IlluminatePaginatorAdapter($collection));
        }
        $rootScope = $this->fractal->createData($resource);
        return $this->respondWithArray($rootScope->toArray());
    }

    protected function respondWithErrors($errors = 'api.error', $statusCode = null, $data = [], $message = null)
    {
        $statusCode = !empty($statusCode) ? $statusCode : self::RESPONSE_STATUS_ERROR;
        if (is_string($errors)) {
            $errors = __($errors);
        }
        $response = ['status' => $statusCode, 'errors' => $errors];
        if (!empty($message)) {
            $response['message'] = $message;
        }
        if (!empty($data)) {
            $response['data'] = $data;
        }
        return $this->setStatusCode($statusCode)->respond($response);
    }



    protected function respondWithBoolean($result)
    {
        return $result ? $this->respondWithSuccess() : $this->errorUnknown();
    }



    //*******************************************************************


    public function respondWithModel($model)
    {
        if ($model) {
            return $this->respondWithObject($model);
        } else {
            return $this->respondwithErrors($model->getErrors());
        }
    }

    public function respondWithObject($obj)
    {
        return $this->respondWithData($obj);
    }


    public function respondWithSuccessMsg($message)
    {
        return $this->setStatusCode(self::RESPONSE_STATUS_OK)->respond(['message' => $message, 'status' => $this->getStatusCode()]);
    }
    public function respondWithErrorMsg($message)
    {
        return $this->setStatusCode(self::RESPONSE_STATUS_ERROR)->respond(['message' => $message, 'status' => $this->getStatusCode()]);
    }


    /**
     * Generates a Response with a 403 HTTP header and a given message.
     *
     * @return Response
     */
    public function errorForbidden($message = 'Forbidden')
    {
        return $this->setStatusCode(403)->respondWithError($message);
    }

    /**
     * Generates a Response with a 500 HTTP header and a given message.
     *
     * @return Response
     */
    public function errorInternalError($message = 'Internal Error')
    {
        return $this->setStatusCode(500)->respondWithError($message);
    }

    /**
     * Generates a Response with a 404 HTTP header and a given message.
     *
     * @return Response
     */
    public function errorNotFound($message = 'Resource Not Found')
    {
        return $this->setStatusCode(404)->respondWithError($message);
    }

    /**
     * Generates a Response with a 401 HTTP header and a given message.
     *
     * @return Response
     */
    public function errorUnauthorized($message = 'Unauthorized')
    {
        return $this->setStatusCode(401)->respondWithError($message);
    }

    /**
     * Generates a Response with a 400 HTTP header and a given message.
     *
     * @return Response
     */
    public function errorWrongArgs($message = 'Wrong Arguments')
    {
        return $this->setStatusCode(400)->respondWithError($message);
    }

    /**
     * Generates a Response with a 500 HTTP header and a given message.
     *
     * @return Response
     */
    public function errorUnknown($message = 'dashboard.unknown_error')
    {
        return $this->setStatusCode(500)->respondWithError($message);
    }

}
